/*
    Extended KMessageWidget for link-local xmpp account autocreation
    Copyright (C) 2011  Martin Klapetek <martin.klapetek@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef SALUT_MESSAGE_WIDGET_H
#define SALUT_MESSAGE_WIDGET_H

#include <KMessageWidget>

class SalutMessageWidget : public KMessageWidget
{
    Q_OBJECT
public:
    SalutMessageWidget(QWidget *parent = 0);
    virtual ~SalutMessageWidget();

    void setParams(const QString &firstname, const QString &lastname, const QString &nickname);

Q_SIGNALS:
    void timeout();
    void configPressed();
    void cancelPressed();
};

#endif // SALUT_MESSAGE_WIDGET_H
